@@ -56,6 +56,7 @@ const float PHASE_SCALE   = 0.5f;
 const float OCTAVE_SCALE  = 12.0f;
 const float TUNING_SCALE  = 1.0f;
 const float SWEEP_SCALE   = 0.5f;
+const float PITCH_SCALE   = 0.5f;
 
 const float LFO_FREQ_MIN  = 0.4f;
 const float LFO_FREQ_MAX  = 40.0f;
@@ -81,8 +82,6 @@ inline float samplv1_sigmoid_0 ( const float x, const float t0 )
 {
 	const float t1 = 1.0f - t0;
 #if 0
-	return (x < -1.0f ? -t1 : (x > +1.0f ? t1 : t1 * x * (1.5f - 0.5f * x * x)));
-#else
 	if (x > +t1)
 		return +t1 + t0 * samplv1_tanhf(+(x - t1) / t0);
 	else
@@ -90,6 +89,8 @@ inline float samplv1_sigmoid_0 ( const float x, const float t0 )
 		return -t1 - t0 * samplv1_tanhf(-(x + t1) / t0);
 	else
 		return x;
+#else
+	return (x < -1.0f ? -t1 : (x > +1.0f ? t1 : t1 * x * (1.5f - 0.5f * x * x)));
 #endif
 }
 
@@ -107,6 +108,17 @@ inline float samplv1_velocity ( const float x, const float p = 0.2f )
 }
 
 
+// pitchbend curve
+
+inline float samplv1_pow2f ( const float x )
+{
+// simplest power-of-2 straight linearization
+// -- x argument valid in [-1, 1] interval
+//	return 1.0f + (x < 0.0f ? 0.5f : 1.0f) * x;
+	return ::powf(2.0f, x);
+}
+
+
 // envelope
 
 struct samplv1_env
@@ -249,10 +261,11 @@ struct samplv1_ctl
 	void reset()
 	{
 		pressure = 0.0f;
-		pitchbend = 0.0f;
+		pitchbend = 1.0f;
 		modwheel = 0.0f;
 		panning = 0.0f;
 		volume = 1.0f;
+		sustain = false;
 	}
 
 	float pressure;
@@ -260,6 +273,7 @@ struct samplv1_ctl
 	float modwheel;
 	float panning;
 	float volume;
+	bool  sustain;
 };
 
 
@@ -645,6 +659,8 @@ struct samplv1_voice : public samplv1_list<samplv1_voice>
 	samplv1_env::State lfo1_env;
 
 	samplv1_glide gen1_glide;					// glides (portamento)
+
+	bool sustain;
 };
 
 
@@ -685,6 +701,7 @@ protected:
 	void allSoundOff();
 	void allControllersOff();
 	void allNotesOff();
+	void allSustainOff();
 
 	samplv1_voice *alloc_voice ()
 	{
@@ -713,9 +730,10 @@ private:
 	samplv1_dcf m_dcf1;
 	samplv1_lfo m_lfo1;
 	samplv1_dca m_dca1;
-	samplv1_def m_def1;
 	samplv1_out m_out1;
 
+	samplv1_def m_def1;
+
 	samplv1_cho m_cho;
 	samplv1_fla m_fla;
 	samplv1_pha m_pha;
@@ -1077,7 +1095,7 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	// note on
 	if (status == 0x90 && value > 0) {
 		samplv1_voice *pv = m_notes[key];
-		if (pv) {
+		if (pv/* && !m_ctl.sustain*/) {
 			// retrigger fast release
 			m_dcf1.env.note_off_fast(&pv->dcf1_env);
 			m_lfo1.env.note_off_fast(&pv->lfo1_env);
@@ -1118,6 +1136,8 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			const float srate = float(m_iSampleRate);
 			float frames = uint32_t(*m_gen1.glide * *m_gen1.glide * srate);
 			pv->gen1_glide.reset(frames, pv->gen1_freq);
+			// sustain
+			pv->sustain = false;
 			// allocated
 			m_notes[key] = pv;
 		}
@@ -1126,6 +1146,9 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	else if (status == 0x80 || (status == 0x90 && value == 0)) {
 		samplv1_voice *pv = m_notes[key];
 		if (pv && pv->note >= 0) {
+			if (m_ctl.sustain)
+				pv->sustain = true;
+			else
 			if (pv->dca1_env.stage != samplv1_env::Release) {
 				m_dca1.env.note_off(&pv->dca1_env);
 				m_dcf1.env.note_off(&pv->dcf1_env);
@@ -1139,7 +1162,7 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 		switch (key) {
 		case 0x01:
 			// modulation wheel (cc#1)
-			m_ctl.modwheel = float(value) / 127.0f;
+			m_ctl.modwheel = *m_def1.modwheel * float(value) / 127.0f;
 			break;
 		case 0x07:
 			// channel volume (cc#7)
@@ -1149,6 +1172,12 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			// channel panning (cc#10)
 			m_ctl.panning = float(value - 64) / 64.0f;
 			break;
+		case 0x40:
+			// sustain/damper pedal (cc#64)
+			if (m_ctl.sustain && value <  64)
+				allSustainOff();
+			m_ctl.sustain = bool(value >= 64);
+			break;
 		case 0x78:
 			// all sound off (cc#120)
 			allSoundOff();
@@ -1165,7 +1194,8 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	}
 	// pitch bend
 	else if (status == 0xe0) {
-		m_ctl.pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+		const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+		m_ctl.pitchbend = samplv1_pow2f(*m_def1.pitchbend * pitchbend);
 	}
 }
 
@@ -1215,6 +1245,26 @@ void samplv1_impl::allNotesOff (void)
 }
 
 
+// all sustained notes off
+
+void samplv1_impl::allSustainOff (void)
+{
+	samplv1_voice *pv = m_play_list.next();
+	while (pv) {
+		if (pv->note >= 0 && pv->sustain) {
+			pv->sustain = false;
+			if (pv->dca1_env.stage != samplv1_env::Release) {
+				m_dca1.env.note_off(&pv->dca1_env);
+				m_dcf1.env.note_off(&pv->dcf1_env);
+				m_lfo1.env.note_off(&pv->lfo1_env);
+				pv->gen1.setLoop(false);
+			}
+		}
+		pv = pv->next();
+	}
+}
+
+
 // all reset clear
 
 void samplv1_impl::reset (void)
@@ -1272,11 +1322,9 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 	// controls
 
-	const float lfo1_rate  = *m_lfo1.rate * *m_lfo1.rate;
-	const float lfo1_freq  = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
-	const float lfo1_pitch = *m_lfo1.pitch * *m_lfo1.pitch;
-	const float modwheel1  = *m_def1.modwheel * (lfo1_pitch + m_ctl.modwheel);
-	const float pitchbend1 = (1.0f + *m_def1.pitchbend * m_ctl.pitchbend);
+	const float lfo1_rate = *m_lfo1.rate * *m_lfo1.rate;
+	const float lfo1_freq = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+	const float modwheel1 = m_ctl.modwheel + PITCH_SCALE * *m_lfo1.pitch;
 
 	if (m_gen1.sample0 != *m_gen1.sample) {
 		m_gen1.sample0  = *m_gen1.sample;
@@ -1336,7 +1384,7 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float lfo1 = pv->lfo1_sample * lfo1_env;
 
 				pv->gen1.next(pv->gen1_freq
-					* (pitchbend1 + modwheel1 * lfo1)
+					* (m_ctl.pitchbend + modwheel1 * lfo1)
 					+ pv->gen1_glide.tick());
 
 				float gen1 = pv->gen1.value(k11);
